<?php

/**
 * @file
 * Contains \Drupal\ces_import\ImportOffers.
 *
 */
namespace Drupal\ces_import;

use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\smallads\Entity\SmallAdInterface;

class ImportOffers extends ImportBase {

  protected $entityTypeId = 'smallad';
  protected $bundle = 'offer';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ces_import_offers_form';
  }

    /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['#redirect'] = \Drupal\Core\Url::fromRoute('entity.smallad.collection');
    return $form;
  }



  public function validateForm(array &$form, FormStateInterface $form_state) {
    $rows = $this->loadCsv($form_state);
    if ($new = $this->newTerms($rows)) {
      drupal_set_message(
        $this->t(
          'The following new terms would be created: @terms',
          ['@terms' => implode(', ', $new)]
        ),
        'warning'
      );
    }
    if (!$form_state->getErrors()) {
      parent::validateForm($form, $form_state);
    }
  }

  function newTerms($rows) {
    $needed_term_names = [];
    foreach ($rows as $rownum => $row) {
      //check for new categories that will need creating
      if (array_key_exists('categories', $row) && strlen($row['categories'])) {
        $needed_term_names[] = trim($row['categories']);
      }
    }
    if ($needed_term_names) {
      $existing_terms = $this->entityTypeManager
        ->getStorage('taxonomy_term')
        ->loadByProperties(['name' => $needed_term_names, 'vid' => SMALLAD_CATEGORIES]);
      $existing_term_names = [];
      foreach ($existing_terms as $term) {
        $existing_term_names[] = $term->label();
      }
      $new_term_names = array_diff(array_unique($needed_term_names), $existing_term_names );
      if ($new_term_names) {
        foreach ($new_term_names as $name) {
          Term::Create(['name' => $name, 'vid' => SMALLAD_CATEGORIES])->save();
        }
        drupal_set_message('Created new terms '.implode(', ', $new_term_names));
      }
    }
  }

  function columns() {
    return [
      'ces_uid',//: user id e.g. abcd1234
      'title',//string
      'categories', //comma separated term names
      'body', //string
      'created', //item created date, format:  "2016-01-15 19:10:33"
      'changed', //last changed date, format:  "2016-01-15 19:10:33"
      'expires', //date, format:  "2016-01-15 19:10:33"
      'scope', //0 means just the owner, 2 means every authenticated user
    ];
  }

  function ces_uidProcess(SmalladInterface $entity, $val, $allfields) {
    $owner = $this->entityTypeManager->getStorage('user')->loadByProperties(['name' => $val]);
    $entity->setOwner(reset($owner));
  }

  function loadCsv($form_state) {
    $csv = parent::loadCsv($form_state);
    foreach ($csv as $rownum => $row) {
      if (!$this->entityTypeManager->getStorage('user')->loadByProperties(['name' => $row['ces_uid']])) {
        drupal_set_message(t('Ad with unknown owner @owner ignored.', ['@owner' => $row['ces_uid']]), 'warning');
        unset($csv[$rownum]);
      }
    }
    return $csv;
  }

  //
  function categoriesprocess(SmalladInterface $entity, $val, $allfields) {
    //suppress the message if the term doesn't exist
    //foreach (explode(',', $val) as $termname) {
    //  $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties(['name' => $termname]);
    //}
    try {
      $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties(['name' => $val]);
      if (!$terms) {
        throw new \Exception('unable to load term with name '.$val);
      }
      $entity->categories->entity = reset($terms);
    }
    catch (\Exception $e) {
      drupal_set_message($e->getMessage());
    }
  }

}