<?php

/**
 * @file
 * Contains \Drupal\cforge\Plugin\DevelGenerate\HamletsUserGenerate.
 * @see https://www.drupal.org/node/2503429 for possible reason why batching doesn't work
 */

namespace Drupal\cforge\Plugin\DevelGenerate;

use Drupal\devel_generate\Plugin\DevelGenerate\UserDevelGenerate;
use Drupal\user\Entity\User;

/**
 * Provides a UserDevelGenerate plugin.
 *
 * @DevelGenerate(
 *   id = "hamlets_user",
 *   label = @Translation("Users friendly "),
 *   description = @Translation("Generate 26 users with English names. Optionally delete current users."),
 *   url = "user",
 *   permission = "administer devel_generate",
 *   settings = {
 *     "kill" = FALSE,
 *     "pass" = ""
 *   }
 * )
 */
class HamletsUserGenerate extends UserDevelGenerate {

  /**
   * The user storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $userStorage;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * {@inheritdoc}
   */
  protected function generateElements(array $values) {
    $kill = $values['kill'];
    $pass = $values['pass'];
    $age = $values['time_range'];
    $roles = array_filter($values['roles']);

    if ($kill) {
      $uids = $this->userStorage->getQuery()
        ->condition('uid', 1, '>')
        ->execute();
      $users = $this->userStorage->loadMultiple($uids);
      $this->userStorage->delete($users);
    }

    $first = ['Alice', 'Barry', 'Cherry', 'Dante', 'Ebeneezer', 'Fanny', 'Garry', 'Harry', 'Irma', 'Jerry', 'Kerry', 'Larry', 'Murray', 'Nancy', 'Oliver', 'Perry', 'Quentin', 'Ruby', 'Sylvester', 'Terry', 'Ursula', 'Veronica', 'William', 'Xanadu', 'Yuri', 'Zoe'];
    $last = ['Archer', 'Barker', 'Cooper', 'Duke', 'Earl', 'Fisher', 'Gardener', 'Hooper', 'Indignado', 'Joyner', 'Kellogg', 'Leach', 'Mason', 'Nurse', 'Offal', 'Parker', 'Quarkson', 'Rake', 'Smith', 'Turner', 'Undertaker', 'Vicar', 'Waterman', 'X', 'Yardley', 'Zapatero'];
    shuffle($first);
    shuffle($last);

    if (empty($roles)) {
      $roles = array(DRUPAL_AUTHENTICATED_RID);
    }

    //before saving accounts we need to switch to user 1 otherwise the wallet name will be wrongly derived
    $user = \Drupal::currentUser()->getAccount();
    \Drupal::currentUser()->setAccount(User::load(1));
    for ($i = 0; $i < 26; $i++) {
      $account = $this->userStorage->create(array(
        'uid' => NULL,
        'name' => $first[$i],
        'pass' => 'a',
        'mail' => $first[$i] . '@example.com',
        'status' => 1,
        'created' => REQUEST_TIME - mt_rand(0, $age),
        'roles' => array_values($roles),
        // A flag to let hook_user_* know that this is a generated user.
        'devel_generate' => TRUE,
      ));

      // Populate all fields with sample values.
      $this->populateFields($account);
      //replace the default content with a sensible name
      $account->firstname->value = $first[$i];
      $account->lastname->value = $last[$i];
      $account->save();
    }
    $this->setMessage('26 users created.');

    \Drupal::currentUser()->setAccount($user);
  }

}
